"""Common types."""

import warnings
from typing import Any

from pydantic import (
    BaseModel as PydanticBaseModel,
    ConfigDict,
    GetCoreSchemaHandler,
    model_validator,
)
from pydantic_core import CoreSchema, core_schema

from .exceptions import ExtraArgumentWarning


class BaseModel(PydanticBaseModel):
    """Base model with extra arguments warning."""

    model_config = ConfigDict(populate_by_name=True)

    @model_validator(mode="before")
    @classmethod
    def _warn_extra(cls, data: Any) -> Any:
        if isinstance(data, dict):
            known_fields = set(cls.model_fields.keys())
            known_fields.update({f.alias for f in cls.model_fields.values() if f.alias})
            for field in data.keys() - known_fields:
                msg = "Unexpected extra argument {} for model {}.{}".format(
                    field, cls.__module__, cls.__name__
                )
                warnings.warn(msg, ExtraArgumentWarning, stacklevel=2)
        return data


class BaseId(str):
    """Identifier base class."""

    @classmethod
    def __get_pydantic_core_schema__(
        cls, source_type: Any, handler: GetCoreSchemaHandler
    ) -> CoreSchema:
        return core_schema.no_info_after_validator_function(cls, handler(str))

    def __repr__(self) -> str:
        return "{}({})".format(self.__class__.__name__, super().__repr__())
