"""Registry types."""

from enum import unique

from pydantic import ConfigDict

from .common import BaseId, BaseModel
from .translations import gettext_lazy as _

try:
    from enum import StrEnum  # type: ignore[attr-defined, unused-ignore]
except ImportError:  # pragma: no cover
    from backports.strenum import StrEnum  # type: ignore[assignment, no-redef, unused-ignore]


class BaseObjectId(BaseId):
    """Registry object identifier base class."""


class BaseObjectHistoryId(BaseId):
    """Registry object history identifier base class."""


class SnapshotId(BaseId):
    """Snapshot id."""


class ContactId(BaseObjectId):
    """Contact id."""


class ContactHistoryId(BaseObjectHistoryId):
    """Contact history id."""


class ContactRef(BaseModel):
    """Contact reference info."""

    model_config = ConfigDict(frozen=True)

    id: ContactId
    handle: str
    name: str = ""
    organization: str = ""


class DomainId(BaseObjectId):
    """Domain id."""


class DomainHistoryId(BaseObjectHistoryId):
    """Domain history id."""


class DomainRef(BaseModel):
    """Domain reference info."""

    model_config = ConfigDict(frozen=True)

    id: DomainId
    fqdn: str


class KeysetId(BaseObjectId):
    """Keyset id."""


class KeysetHistoryId(BaseObjectHistoryId):
    """Keyset history id."""


class KeysetRef(BaseModel):
    """Keyset reference info."""

    model_config = ConfigDict(frozen=True)

    id: KeysetId
    handle: str


class NssetId(BaseObjectId):
    """Nsset id."""


class NssetHistoryId(BaseObjectHistoryId):
    """Nsset history id."""


class NssetRef(BaseModel):
    """Nsset reference info."""

    model_config = ConfigDict(frozen=True)

    id: NssetId
    handle: str


class RegistrarId(BaseId):
    """Registrar identifier."""


class RegistrarRef(BaseModel):
    """Registrar reference info."""

    model_config = ConfigDict(frozen=True)

    id: RegistrarId
    handle: str
    name: str = ""
    organization: str = ""


class RepresentativeId(BaseObjectId):
    """Contact representative id."""


class RepresentativeHistoryId(BaseObjectHistoryId):
    """Contact representative history id."""


@unique
class RegistryObjectType(StrEnum):
    """Registry object type."""

    CONTACT: "RegistryObjectType" = "contact"  # type: ignore[assignment,misc,unused-ignore]
    DOMAIN: "RegistryObjectType" = "domain"  # type: ignore[assignment,misc,unused-ignore]
    KEYSET: "RegistryObjectType" = "keyset"  # type: ignore[assignment,misc,unused-ignore]
    NSSET: "RegistryObjectType" = "nsset"  # type: ignore[assignment,misc,unused-ignore]


@unique
class StateFlag(StrEnum):
    """Enum of object state flags."""

    label: str

    def __new__(cls, value: str, label: str = "") -> "StateFlag":
        """Construct item with label."""
        obj = str.__new__(cls, value)
        obj._value_ = value

        obj.label = label
        return obj

    CONDITIONALLY_IDENTIFIED_CONTACT: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "conditionallyIdentifiedContact",
        _("Contact is conditionally identified"),
    )
    DELETE_CANDIDATE: "StateFlag" = ("deleteCandidate", _("To be deleted"))  # type: ignore[assignment,misc,unused-ignore]
    DELETE_WARNING: "StateFlag" = ("deleteWarning", _("The domain will be deleted in 11 days"))  # type: ignore[assignment,misc,unused-ignore]
    EXPIRATION_WARNING: "StateFlag" = ("expirationWarning", _("The domain expires in 30 days"))  # type: ignore[assignment,misc,unused-ignore]
    EXPIRED: "StateFlag" = ("expired", _("Domain expired"))  # type: ignore[assignment,misc,unused-ignore]
    IDENTIFIED_CONTACT: "StateFlag" = ("identifiedContact", _("Contact is identified"))  # type: ignore[assignment,misc,unused-ignore]
    LINKED: "StateFlag" = ("linked", _("Has relation to other records in the registry"))  # type: ignore[assignment,misc,unused-ignore]
    NOT_VALIDATED: "StateFlag" = ("notValidated", _("Domain not validated"))  # type: ignore[assignment,misc,unused-ignore]
    NSSET_MISSING: "StateFlag" = ("nssetMissing", _("The domain doesn't have associated nsset"))  # type: ignore[assignment,misc,unused-ignore]
    OUTZONE: "StateFlag" = ("outzone", _("The domain isn't generated in the zone"))  # type: ignore[assignment,misc,unused-ignore]
    OUTZONE_UNGUARDED: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "outzoneUnguarded",
        _("The domain is out of zone after 30 days in expiration state"),
    )
    OUTZONE_UNGUARDED_WARNING: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "outzoneUnguardedWarning",
        _("The domain is to be out of zone soon."),
    )
    PREMIUM_DOMAIN: "StateFlag" = ("premiumDomain", _("VIP domain"))  # type: ignore[assignment,misc,unused-ignore]
    SERVER_BLOCKED: "StateFlag" = ("serverBlocked", _("Administratively blocked"))  # type: ignore[assignment,misc,unused-ignore]
    SERVER_CONTACT_IDENT_CHANGE_PROHIBITED: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "serverContactIdentChangeProhibited",
        _("Ident update forbidden"),
    )
    SERVER_CONTACT_NAME_CHANGE_PROHIBITED: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "serverContactNameChangeProhibited",
        _("Name update forbidden"),
    )
    SERVER_CONTACT_ORGANIZATION_CHANGE_PROHIBITED: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "serverContactOrganizationChangeProhibited",
        _("Organization update forbidden"),
    )
    SERVER_CONTACT_PERMANENT_ADDRESS_CHANGE_PROHIBITED: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "serverContactPermanentAddressChangeProhibited",
        _("Permanent address update forbidden"),
    )
    SERVER_DELETE_PROHIBITED: "StateFlag" = ("serverDeleteProhibited", _("Deletion forbidden"))  # type: ignore[assignment,misc,unused-ignore]
    SERVER_INZONE_MANUAL: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "serverInzoneManual",
        _("The domain is administratively kept in zone"),
    )
    SERVER_LINK_PROHIBITED: "StateFlag" = ("serverLinkProhibited", _("Contact linking forbidden"))  # type: ignore[assignment,misc,unused-ignore]
    SERVER_OUTZONE_MANUAL: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "serverOutzoneManual",
        _("The domain is administratively kept out of zone"),
    )
    SERVER_REGISTRANT_CHANGE_PROHIBITED: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "serverRegistrantChangeProhibited",
        _("Registrant change forbidden"),
    )
    SERVER_RENEW_PROHIBITED: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "serverRenewProhibited",
        _("Registration renewal forbidden"),
    )
    SERVER_TRANSFER_PROHIBITED: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "serverTransferProhibited",
        _("Sponsoring registrar change forbidden"),
    )
    SERVER_UPDATE_PROHIBITED: "StateFlag" = ("serverUpdateProhibited", _("Update forbidden"))  # type: ignore[assignment,misc,unused-ignore]
    UNGUARDED: "StateFlag" = ("unguarded", _("The domain is 30 days after expiration"))  # type: ignore[assignment,misc,unused-ignore]
    VALIDATED_CONTACT: "StateFlag" = ("validatedContact", _("Contact is validated"))  # type: ignore[assignment,misc,unused-ignore]
    VALIDATION_WARNING1: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "validationWarning1",
        _("The domain validation expires in 30 days"),
    )
    VALIDATION_WARNING2: "StateFlag" = (  # type: ignore[assignment,misc,unused-ignore]
        "validationWarning2",
        _("The domain validation expires in 15 days"),
    )
